@@ -1,6 +1,5 @@
-import json
-
 import numpy as np
+import tensorflow as tf
 
 AA_DICT = {'?': 0, '<': 1, '>': 2,
            'A': 3, 'C': 4, 'D': 5, 'E': 6, 'F': 7,
@@ -10,7 +9,8 @@ AA_DICT = {'?': 0, '<': 1, '>': 2,
 
 AA_DICT_REV = {AA_DICT[key]: key for key in AA_DICT}
 
-AA_DISTRIBUTION = np.load('prot2vec/configs/probab_distribution.npy')
+
+# AA_DISTRIBUTION = np.load('prot2vec/configs/probab_distribution.npy')
 
 
 def tokenize(seq):
@@ -23,15 +23,22 @@ def tokenize(seq):
 
     digit_seq = np.array(digit_seq)
 
-    digit_seq[0] = 1
-    digit_seq[-1] = 2
+    # TODO-WARNING: resolve this issue!
+    # digit_seq[0] = 1
+    # digit_seq[-1] = 2
 
     return digit_seq[None, :]
 
 
-def mask_data(data, mask_prob):
-    bool_mask = np.random.rand(data.shape[0] * data.shape[1]) < mask_prob
-    bool_mask = bool_mask.reshape(data.shape)
+def mask_data(data, mask_prob, one_in_middle):
+    if one_in_middle:
+        bool_mask = np.zeros(data.shape, dtype='bool')
+        mask_index = bool_mask.shape[1] // 2
+        bool_mask[:, mask_index] = True
+    else:
+        bool_mask = np.random.rand(data.shape[0] * data.shape[1]) < mask_prob
+        bool_mask = bool_mask.reshape(data.shape)
+
     bool_mask[:, 0] = False
     bool_mask[:, -1] = False
 
@@ -44,13 +51,13 @@ def mask_data(data, mask_prob):
     return data, bool_mask
 
 
-def vect_batch(batch, mask_prob):
+def vect_batch(batch, mask_prob, one_in_middle):
     x = np.concatenate(batch, axis=0)
     y = x.copy()
 
-    x, mask = mask_data(x, mask_prob)
+    x, mask = mask_data(x, mask_prob, one_in_middle)
 
-    return x, y, mask
+    return tf.one_hot(x, len(AA_DICT)), tf.one_hot(y, len(AA_DICT)), mask
 
 
 def random_iter(seq_len, masked_size, batch_size, overfit_batch=False):
@@ -74,10 +81,11 @@ def random_iter(seq_len, masked_size, batch_size, overfit_batch=False):
         yield seq_in, seq_out
 
 
-def ds_iter(src_file, seq_len, mask_prob, batch_size):
+def ds_iter(src_file, seq_len, mask_prob, batch_size, one_in_middle=False):
     batch = []
     while True:
         with open(src_file, "r", encoding="utf-8") as fr:
+            fr.readline()
             for line in fr:
                 _, _, seq = line[:-1].split('\t')
                 prot_len = len(seq[:-1])
@@ -95,6 +103,20 @@ def ds_iter(src_file, seq_len, mask_prob, batch_size):
                 batch.append(seq_in)
 
                 if len(batch) >= batch_size:
-                    v_batch = vect_batch(batch, mask_prob)
+                    v_batch = vect_batch(batch, mask_prob, one_in_middle)
                     yield v_batch
                     del batch[:]
+
+
+if __name__ == '__main__':
+    from pathlib import Path
+
+    ROOT = Path(r'C:\DATA\ML-Data\BioML\datasets\UniRef90')
+    ds_it = ds_iter(ROOT / 'refprot_random_taxfree.tsv', 64, 0.15, 1024, True)
+    a = next(ds_it)
+    a = next(ds_it)
+    a = next(ds_it)
+    # start = time.time()
+    for i in range(1000):
+        a = next(ds_it)
+    # print(time.time() - start)
